/*---[ netfilter-script.c ]-------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Functions to write the netfilter shell scripts
 *--------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "druid.h"
#include "netfilter-script.h"

/* [ write_netfilter_script ]
 * Creates the netfilter shell script
 */
void
write_netfilter_script (Druid *data)
{
	gboolean masq = GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (data->masq));

	gchar *scriptpath = FIRESTARTER_RULES_DIR "/firestarter/firewall.sh";

	FILE *script = fopen (scriptpath, "w");

	chmod (scriptpath, 00700);

/* Init variables and external files */
	fprintf (script, "#!/bin/sh\n");
	fprintf (script, "#Generated by Firestarter " VERSION ", NETFILTER in use\n\n");

	if (gnome_config_get_bool ("/firestarter/Druid/locatesbins=FALSE"))
		fprintf (script, "IPT=`which iptables`\n");
	else
		fprintf (script, "IPT=/sbin/iptables\n");
	/* The network interface */
	fprintf (script, "IF=%s\n", gtk_entry_get_text (GTK_ENTRY (data->extdevice)));
	/* The internal network interface */
	if (masq)
		fprintf (script, "INIF=%s\n", gtk_entry_get_text (GTK_ENTRY (data->intdevice)));

	/* IP address of interface */
	fprintf (script, "IP=`/sbin/ifconfig $IF | grep inet | cut -d : -f 2 | cut -d \\  -f 1`\n");
	/* Netmask/net of external interface */
	fprintf (script, "MASK=`/sbin/ifconfig $IF | grep Mas | cut -d : -f 4`\n");
	fprintf (script, "NET=$IP/$MASK\n");
	/* Internal network*/
	if (masq) {
		if (GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (data->auto_int_ip)))
			fprintf (script, "INNET=%s\n\n", gtk_entry_get_text (GTK_ENTRY (data->intrange)));
		else {
			fprintf (script, "INIP=`/sbin/ifconfig $INIF | grep inet | cut -d : -f 2 | cut -d \\  -f 1`\n");
			fprintf (script, "INMASK=`/sbin/ifconfig $INIF | grep Mas | cut -d : -f 4`\n");
			fprintf (script, "INNET=$INIP/$INMASK\n");
		}
	}

	fprintf (script, "#Delete user made chains. Flush and zero the chains.\n");
	fprintf (script, "$IPT -F\n$IPT -X\n$IPT -Z\n\n");

	fprintf (script, "#Delete `nat' and `mangle' chains.\n");
	fprintf (script, "$IPT -t nat -F\n$IPT -t mangle -F\n\n");

	fprintf (script, "#Create a new log and drop (LD) convenience chain.\n");
	fprintf (script, "$IPT -N LD\n$IPT -A LD -j LOG\n$IPT -A LD -j DROP\n\n");

	/* Pipe in the modrules files */
	fprintf (script, "#Deny deny all traffic on these ports, without logging\n");
	fprintf (script, "if [ -e "FIRESTARTER_RULES_DIR "/firestarter/do-not-log-ports ]\n then\n"
		"source " FIRESTARTER_RULES_DIR "/firestarter/do-not-log-ports\nfi\n");

	fprintf (script, "#Deny all traffic from these machines\n");
	fprintf (script, "source " FIRESTARTER_RULES_DIR "/firestarter/deny-all\n");
	fprintf (script, "#Allow all traffic from these machines\n");
	fprintf (script, "source " FIRESTARTER_RULES_DIR "/firestarter/allow-all\n");
	fprintf (script, "#Allow a specific service to a specific machine\n");
	fprintf (script, "source " FIRESTARTER_RULES_DIR "/firestarter/allow-service-machine\n");
	fprintf (script, "#Allow a specific service to everyone\n");
	fprintf (script, "source " FIRESTARTER_RULES_DIR "/firestarter/allow-service-all\n\n");

/* GOOD IDEA(tm) RULES */
	fprintf (script, "#Allow all traffic on the loopback interface\n");
	fprintf (script, "$IPT -t filter -A INPUT -i lo -s 0/0 -d 0/0 -j ACCEPT\n");
	fprintf (script, "$IPT -t filter -A OUTPUT -o lo -s 0/0 -d 0/0 -j ACCEPT\n\n");

	fprintf (script, "#Turn on source address verification in kernel\n");	
	fprintf (script, "if [ -e /proc/sys/net/ipv4/conf/all/rp_filter ]\n then\n"
	"  for f in /proc/sys/net/ipv4/conf/*/rp_filter\n  do\n   echo 2 > $f\n  done\nfi\n\n");

	fprintf (script, "#Turn on syn cookies protection in kernel\n");	
	fprintf (script, "if [ -e /proc/sys/net/ipv4/tcp_syncookies ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/tcp_syncookies\nfi\n\n");

	fprintf (script, "#ICMP Dead Error Messages protection\n");
	fprintf (script, "if [ -e /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses\nfi\n\n");
	
	fprintf (script, "#ICMP Broadcasting protection\n");
	fprintf (script, "if [ -e /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts\nfi\n\n");
	
	fprintf (script, "#Turn on dynamic TCP/IP address hacking\n");
	fprintf (script, "if [ -e /proc/sys/net/ipv4/ip_dynaddr ]\n then\n"
	"  echo 1 > /proc/sys/net/ipv4/ip_dynaddr\nfi\n\n");

/* ICMP RULES */

if GTK_WIDGET_IS_SENSITIVE (data->icmp)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_echo))) {
		fprintf (script, "#ICMP: Ping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type echo-request -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Ping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type echo-request -m limit --limit 1/s -j ACCEPT -l\n");
	}
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_traceroute))) {
		fprintf (script, "#ICMP: Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 33434 -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 33434 -j ACCEPT -l\n");
	}
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_tracert))) {
		fprintf (script, "#ICMP: MS Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type destination-unreachable -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: MS Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type destination-unreachable -j ACCEPT -l\n");
	}
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_unreach))) {
		fprintf (script, "#ICMP: Unreachable Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type host-unreachable -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Unreachable Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type host-unreachable -j ACCEPT -l\n");
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_timestamp))) {
		fprintf (script, "#ICMP: Timestamping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-request -j REJECT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-reply -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Timestamping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-request -j ACCEPT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-reply -j ACCEPT -l\n");
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_masking))) {
		fprintf (script, "#ICMP: Address Masking\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-request -j REJECT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-reply -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Address Masking\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-request -j ACCEPT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-reply -j ACCEPT -l\n");
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_redir))) {
		fprintf (script, "#ICMP: Redirection Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type redirect -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Redirection Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type redirect -m limit --limit 2/s -j ACCEPT -l\n");
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_quench))) {
		fprintf (script, "#ICMP: Source Quench Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type source-quench -j REJECT -l\n");
	}
		else {
		fprintf (script, "#ICMP: Source Quench Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type source-quench -m limit --limit 2/s -j ACCEPT -l\n");
	}
}
else {
		fprintf (script, "#ICMP: Ping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type echo-request -m limit --limit 1/s -j ACCEPT -l\n");
		fprintf (script, "#ICMP: Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET 33434 -j REJECT -l\n");
		fprintf (script, "#ICMP: MS Traceroute Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type destination-unreachable -j REJECT -l\n");
		fprintf (script, "#ICMP: Unreachable Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type host-unreachable -j REJECT -l\n");
		fprintf (script, "#ICMP: Timestamping Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-request -j REJECT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type timestamp-reply -j REJECT -l\n");
		fprintf (script, "#ICMP: Address Masking\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-request -j REJECT -l\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type address-mask-reply -j REJECT -l\n");
		fprintf (script, "#ICMP: Redirection Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type redirect -j REJECT -l\n");
		fprintf (script, "#ICMP: Source Quench Requests\n");
		fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET --icmp-type source-quench -m limit --limit 2/s -j ACCEPT -l\n");
}

/* MASQ RULES */
	if (masq) {
		fprintf (script, "#Turn on IP forwarding\n");
		fprintf (script, "if [ -e /proc/sys/net/ipv4/ip_forward ]\n then\n"
		"  echo 1 > /proc/sys/net/ipv4/ip_forward\nfi\n\n");

		fprintf (script, "#Load iptable NAT & Mangle modules\n");
		fprintf (script, "/sbin/modprobe iptable_nat\n");
		fprintf (script, "/sbin/modprobe iptable_mangle\n\n");

		fprintf (script, "Forward Int/Ext & Ext/Int Traffic before Masquerading\n");
		fprintf (script, "$IPT -t filter -A FORWARD -d 0/0 -s $INNET -o $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A FORWARD -d $INNET -s $NET -i $IF -j ACCEPT\n");

		fprintf (script, "#Masquerade outgoing traffic\n");
		fprintf (script, "$IPT -t nat -A POSTROUTING -o $IF -j MASQUERADE\n\n");

		fprintf (script, "#Don't masq external interface traffic\n");
		fprintf (script, "$IPT -t nat -A POSTROUTING -s $NET -d 0/0 -j ACCEPT\n\n");

		fprintf (script, "#Allow traffic from internal network going anywhere\n");
		fprintf (script, "$IPT -t filter -A INPUT -s $INNET -d 0/0 -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A OUTPUT -s $INNET -d 0/0 -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A OUTPUT -p icmp -s $INNET -d 0/0 -j ACCEPT\n\n");

		fprintf (script, "Setting default forwarding rule to DROP\n");
		fprintf (script, "$IPT -t filter -P FORWARD DROP\n\n");
		
	}

	fprintf (script, "#Block nonroutable IPs\n");	
	fprintf (script, "$IPT -t filter -A INPUT -s 127.0.0.0/8 -d $NET -i $IF -j DROP\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 128.66.0.0/16 -d $NET -i $IF -j DROP\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 172.16.0.0/12 -d $NET -i $IF -j DROP\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 192.168.0.0/16 -d $NET -i $IF -j DROP\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 197.0.0.0/16 -d $NET -i $IF -j DROP\n\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 201.0.0.0/8 -d $NET -i $IF -j DROP\n\n");

	fprintf (script, "#Block Back Orifice\n");	
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 31337 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 31337 -j LD\n\n");

	fprintf (script, "#Block NetBus\n");	
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 12345:12346 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 12345:12346 -j LD\n\n");

	fprintf (script, "#Block Trin00\n");	
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 1524 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 27665 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 27444 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 31335 -j LD\n\n");

	fprintf (script, "#Block Multicast\n");	
	fprintf (script, "$IPT -t filter -A INPUT -s 224.0.0.0/8 -d 0/0 -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -s 0/0 -d 224.0.0.0/8 -j LD\n\n");

/* RULES BASED ON WIZARD SELECTIONS */

	/* Allow DHCP */
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dhcpcheck))) {
		fprintf (script, "#DHCP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 67:68 -i $IF -j ACCEPT\n\n");
	}

if GTK_WIDGET_IS_SENSITIVE (data->services)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ftp))) {
		fprintf (script, "#FTP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 20  ! --syn -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 21 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ssh))) {
		fprintf (script, "#SSH\n");		
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 22 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->telnet))) {
		fprintf (script, "#Telnet\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 23 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->smtp))) {
		fprintf (script, "#SMTP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 25 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dns))) {
		fprintf (script, "#DNS\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 53 -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 53 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->finger))) {
		fprintf (script, "#Finger\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 79 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->www))) {
		fprintf (script, "#HTTP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 80 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->sslweb))) {
		fprintf (script, "#SSL HTTP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 443 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->pop))) {
		fprintf (script, "#POP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 110 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->imap))) {
		fprintf (script, "#IMAP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 143 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ident))) {
		fprintf (script, "#IDENT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 113 -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 113 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nntp))) {
		fprintf (script, "#NNTP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 119 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ntp))) {
		fprintf (script, "#NTP\n");		
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d $NET --dport 123 -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 123 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->samba))) {
		fprintf (script, "#SAMBA\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 445 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 445 -i $IF -j ACCEPT\n\n");
	}
 	else {
		fprintf (script, "#Block SAMBA\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j LD\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j LD\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 445 -i $IF -j LD\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 445 -i $IF -j LD\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->routed))) {
		fprintf (script, "#Routed\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 520 -j ACCEPT\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nfs))) {
		fprintf (script, "#NFS\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 2049 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 2049 -i $IF -j ACCEPT\n\n");
	}
	else {
		fprintf (script, "#Block NFS\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 2049 -i $IF -j LD\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 2049 -i $IF -j LD\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->x))) {
		fprintf (script, "#Xwindows\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j ACCEPT\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j ACCEPT\n\n");
	}
	else {
		fprintf (script, "#Block Xwindows\n");
		fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j LD\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j LD\n\n");
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dhcp))) {
		fprintf (script, "#DHCP\n");
		fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 67:68 -i $IF -j ACCEPT\n\n");
	}
}
else {
	fprintf (script, "#Block NFS\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 2049 -i $IF -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 2049 -i $IF -j LD\n\n");
	
	fprintf (script, "#Block SAMBA\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 137:139 -i $IF -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 445 -i $IF -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 445 -i $IF -j LD\n\n");

	fprintf (script, "#Block Xwindows\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j LD\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d 0/0 --dport 6000:6010 -i $IF -j LD\n\n");
}


/* REQUIRED RULES */
	fprintf (script, "#Allow ICMP\n");
	fprintf (script, "$IPT -t filter -A INPUT -p icmp -s 0/0 -d $NET -j ACCEPT\n");
	fprintf (script, "$IPT -t filter -A OUTPUT -p icmp -s $NET -d 0/0 -j ACCEPT\n\n");

	fprintf (script, "#Open ports for established connections\n");
	fprintf (script, "$IPT -t filter -A INPUT -p tcp ! --syn -s 0/0 -d $NET --dport 1023:65535 -j ACCEPT\n");
	fprintf (script, "$IPT -t filter -A INPUT -p udp -s 0/0 -d $NET --dport 1023:65535 -j ACCEPT\n\n");

	fprintf (script, "#Deny everything not let through earlier\n");
	fprintf (script, "$IPT -A OUTPUT -j ACCEPT\n");
	fprintf (script, "$IPT -A INPUT -j LD\n");

	fclose (script);

	g_print (_("Firewall script saved as %s\n"), scriptpath);
}
